#ifndef __BOT_H
#define __BOT_H

#include <gloox/client.h>
#include <gloox/messagehandler.h>
#include <gloox/connectionlistener.h>
#include <gloox/rostermanager.h>
#include <gloox/rosterlistener.h>
#include <command.h>

using namespace gloox;

class Bot : public ConnectionListener, RosterListener, MessageHandler
{
 private:
    std::string mJid;
    std::string mPass;
    std::string mHost;
    Client * mClient;
    RosterManager * mRosterManager;
    CommandManager * mCommandManager;
    void sendReply(const JID &to, const std::string &body);

 public:
    Bot(std::string jid, std::string pass, std::string host);
    virtual ~Bot();
    void start(void);
    CommandManager * commandManager(void);

    // gloox::ConnectionListener
    virtual void onConnect();
    virtual void onDisconnect(gloox::ConnectionError e);
    virtual bool onTLSConnect(const CertInfo& info);

    // gloox::RosterListener
    virtual void handleItemAdded (const JID &jid);
    virtual void handleItemSubscribed (const JID &jid);
    virtual void handleItemRemoved (const JID &jid);
    virtual void handleItemUpdated (const JID &jid);
    virtual void handleItemUnsubscribed (const JID &jid);
    virtual void handleRoster (const Roster &roster);
    virtual void handleRosterPresence (const RosterItem &item, const std::string &resource, Presence presence, const std::string &msg);
    virtual void handleSelfPresence (const RosterItem &item, const std::string &resource, Presence presence, const std::string &msg);
    virtual bool handleSubscriptionRequest (const JID &jid, const std::string &msg);
    virtual bool handleUnsubscriptionRequest (const JID &jid, const std::string &msg);
    virtual void handleNonrosterPresence (Stanza *stanza);
    virtual void handleRosterError (Stanza *stanza);

    // gloox::MessageHandler
    virtual void handleMessage(Stanza * stanza, MessageSession * session);
};

#endif
